@@ -1,26 +1,27 @@
 ;;; closql.el --- store EIEIO objects using EmacSQL  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2020  Jonas Bernoulli
+;; Copyright (C) 2016-2021  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Homepage: https://github.com/emacscollective/closql
-;; Package-Requires: ((emacs "25.1") (emacsql-sqlite "3.0.0"))
 ;; Keywords: extensions
-
-;; This file is not part of GNU Emacs.
+;; Package-Requires: ((emacs "25.1") (emacsql-sqlite "3.0.0"))
+;; SPDX-License-Identifier: GPL-3.0-or-later
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published
 ;; by the Free Software Foundation; either version 3 of the License,
 ;; or (at your option) any later version.
-
+;;
 ;; This file is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
-
+;;
 ;; For a full copy of the GNU GPL see http://www.gnu.org/licenses.
 
+;; This file is not part of GNU Emacs.
+
 ;;; Commentary:
 
 ;; Store uniform EIEIO objects in an EmacSQL database.  SQLite is used
@@ -36,6 +37,9 @@
 
 (eval-when-compile (require 'subr-x))
 
+(unless (boundp 'eieio--unbound) ; New name since Emacs 28.1.
+  (defvaralias 'eieio--unbound 'eieio-unbound nil))
+
 ;;; Objects
 
 (defclass closql-object ()
@@ -106,7 +110,7 @@
                                   obj (oref-default obj closql-primary-key))
                                  (oref-default class closql-primary-key)))))
          (table
-          (if (eq value eieio-unbound)
+          (if (eq value eieio--unbound)
               (let ((columns (closql--table-columns db table)))
                 (aset obj c
                       (mapcar
@@ -148,7 +152,7 @@
       (eieio--validate-slot-value class c value slot)
       (unless (eq slot 'closql-database)
         (let ((db (closql--oref obj 'closql-database)))
-          (unless (or (not db) (eq db eieio-unbound))
+          (unless (or (not db) (eq db eieio--unbound))
             (closql--dset db obj slot value))))
       (aset obj c value))))
 
@@ -164,7 +168,7 @@
       (emacsql-with-transaction db
         (let ((columns (closql--table-columns db table)))
           ;; Caller might have modified value in place.
-          (closql--oset obj slot eieio-unbound)
+          (closql--oset obj slot eieio--unbound)
           (let ((list1 (closql-oref obj slot))
                 (list2 value)
                 elt1 elt2)
@@ -212,7 +216,7 @@
       (emacsql db [:update $i1 :set (= $i2 $s3) :where (= $i4 $s5)]
                (oref-default obj closql-table)
                slot
-               (if (eq value eieio-unbound) 'eieio-unbound value)
+               (if (eq value eieio--unbound) 'eieio-unbound value)
                key id)))))
 
 ;;;; Slot Properties
@@ -269,8 +273,9 @@
 
 ;;; Database
 
-(defclass closql-database (emacsql-sqlite-connection)
-  ((object-class :allocation :class)))
+(defclass closql-database ()
+  ((object-class :allocation :class))
+  :abstract t)
 
 (cl-defmethod closql-db ((class (subclass closql-database))
                          &optional variable file debug)
@@ -279,10 +284,11 @@
              (prog1 db (emacsql db [:pragma (= foreign-keys on)]))))
       (let ((db-init (not (and file (file-exists-p file))))
             (db (make-instance class :file file)))
-        (set-process-query-on-exit-flag (oref db process) nil)
+        (when (slot-boundp db 'process)
+          (set-process-query-on-exit-flag (oref db process) nil))
         (when debug
           (emacsql-enable-debugging db))
-        (emacsql db (emacsql db [:pragma (= foreign-keys on)]))
+        (emacsql db [:pragma (= foreign-keys on)])
         (when db-init
           (closql--db-init db))
         (when variable
@@ -311,7 +317,7 @@
       (let ((table (closql--slot-table obj slot)))
         (when table
           (push (cons slot (closql-oref obj slot)) alist)
-          (closql--oset obj slot eieio-unbound))))
+          (closql--oset obj slot eieio--unbound))))
     (emacsql-with-transaction db
       (emacsql db
                (if replace
@@ -344,7 +350,7 @@
                   (eieio-object-class obj))
       (error "Cannot reload object")))
 
-(cl-defmethod closql-get ((db closql-database) ident &optional class)
+(cl-defmethod closql-get ((db closql-database) ident &optional class resolve)
   (unless class
     (setq class (oref-default db object-class)))
   (when-let ((row (car (emacsql db [:select * :from $i1
@@ -352,7 +358,7 @@
                                 (oref-default class closql-table)
                                 (oref-default class closql-primary-key)
                                 ident))))
-    (closql--remake-instance class db row t)))
+    (closql--remake-instance class db row resolve)))
 
 (cl-defmethod closql-query ((db closql-database) &optional select pred class)
   (if select
@@ -428,12 +434,12 @@
 
 (defun closql--intern-unbound (row)
   (mapcar (lambda (elt)
-            (if (eq elt eieio-unbound) 'eieio-unbound elt))
+            (if (eq elt eieio--unbound) 'eieio-unbound elt))
           row))
 
 (defun closql--extern-unbound (row)
   (mapcar (lambda (elt)
-            (if (eq elt 'eieio-unbound) eieio-unbound elt))
+            (if (eq elt 'eieio-unbound) eieio--unbound elt))
           row))
 
 (defun closql--coerce (object type)
@@ -541,14 +547,14 @@
     (emacsql db (format "\
 SELECT DISTINCT %s FROM %s AS d, %s AS i
 WHERE d.%s = i.%s AND d.%s = '%S';"
-                (mapconcat (apply-partially #'format "i.%s")
-                           (cddr i-cols) ", ")
-                d-table
-                i-table
-                (cadr d-cols)
-                (cadr i-cols)
-                (car  d-cols)
-                obj-id))))
+                        (mapconcat (apply-partially #'format "i.%s")
+                                   (cddr i-cols) ", ")
+                        d-table
+                        i-table
+                        (cadr d-cols)
+                        (cadr i-cols)
+                        (car  d-cols)
+                        obj-id))))
 
 (defun closql--slot-tables (obj slot)
   (let ((tbls (closql--slot-get obj slot :closql-table)))
